/*

    Libre Media Server Audio - An Open source Media Server for arts and performing.
    (c) Criptomart - Santiago Noreña 2012-2024 <lms@criptomart.net>
    https://git.criptomart.net/libremediaserver

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "libremediaserver-audio-ui.h"


libreMediaServerAudioUi::libreMediaServerAudioUi(QWidget *parent)
    : QMainWindow(parent)
{
    ui.setupUi(this);
    this->setWindowTitle(VERSION);
    m_aw = new AudioWidget(this);
    setCentralWidget(m_aw);
    m_dmxWidget = new dmxWidget(this);
    QDockWidget *topWidget = new QDockWidget(tr("Master"), this);
    topWidget->setAllowedAreas(Qt::TopDockWidgetArea);
    topWidget->setWidget(m_dmxWidget);
    topWidget->setContentsMargins(0, 0, 0, 0);
    addDockWidget(Qt::TopDockWidgetArea, topWidget);
    connect(ui.actionLaunch_OLA_Setup, SIGNAL(triggered()), this, SLOT(olasetup()));
    connect(ui.actionLaunchShowPlayer, SIGNAL(triggered()), this, SLOT(launchShowPlayerWindow()));
    this->setContentsMargins(0, 0, 0, 0);
    this->setStyleSheet(
                "margin: 0px;"
                "color: white;"
                "background-color: #3f3038;"
                "selection-color: blue;"
                "selection-background-color: green"
                );
    m_showPlayer = new ShowPlayer();
    m_showPlayer->setFocus();
    QShortcut *shortcut_space = new QShortcut(QKeySequence(" "), this);
    QObject::connect(shortcut_space, SIGNAL(activated()), m_showPlayer, SLOT(go()));
    QShortcut *shortcut_up = new QShortcut(QKeySequence("Up"), this);
    QObject::connect(shortcut_up, SIGNAL(activated()), m_showPlayer->ui->cueListWidget, SLOT(key_up()));
        QShortcut *shortcut_down = new QShortcut(QKeySequence("Down"), this);
        QObject::connect(shortcut_down, SIGNAL(activated()), m_showPlayer->ui->cueListWidget, SLOT(key_down()));
        QShortcut *shortcut_copy = new QShortcut(QKeySequence("Ctrl+C"), this);
        QObject::connect(shortcut_copy, SIGNAL(activated()), m_showPlayer->ui->cueListWidget, SLOT(copyCueTrack()));
        QShortcut *shortcut_cut = new QShortcut(QKeySequence("Ctrl+X"), this);
        QObject::connect(shortcut_cut, SIGNAL(activated()), m_showPlayer->ui->cueListWidget, SLOT(cutCueTrack()));
        QShortcut *shortcut_paste = new QShortcut(QKeySequence("Ctrl+V"), this);
        QObject::connect(shortcut_paste, SIGNAL(activated()), m_showPlayer->ui->cueListWidget, SLOT(pasteCueTrack()));
        QShortcut *shortcut_delete = new QShortcut(QKeySequence("Delete"), this);
        QObject::connect(shortcut_delete, SIGNAL(activated()), m_showPlayer->ui->cueListWidget, SLOT(deleteCueTrack()));
        QShortcut *shortcut_insert = new QShortcut(QKeySequence("Insert"), this);
        QObject::connect(shortcut_insert, SIGNAL(activated()), m_showPlayer->ui->cueListWidget, SLOT(createNewCueTrack()));
        QShortcut *shortcut_edit = new QShortcut(QKeySequence("e"), this);
        QObject::connect(shortcut_edit, SIGNAL(activated()), m_showPlayer->ui->cueListWidget, SLOT(editCueTrack()));
    Settings::getInstance()->beginGroup("mainwindow");
    auto geometry = Settings::getInstance()->value("geometry", QByteArray()).toByteArray();
    if (!geometry.isEmpty())
        restoreGeometry(geometry);
    geometry = Settings::getInstance()->value("showPlayerGeometry", QByteArray()).toByteArray();
    if (!geometry.isEmpty())
        m_showPlayer->restoreGeometry(geometry);
    Settings::getInstance()->endGroup();
}

libreMediaServerAudioUi::~libreMediaServerAudioUi()
{
    Settings::getInstance()->beginGroup("mainwindow");
    Settings::getInstance()->setValue("size", size());
    Settings::getInstance()->setValue("fullScreen", isFullScreen());
    Settings::getInstance()->setValue("geometry", saveGeometry());
    Settings::getInstance()->setValue("showPlayer", m_showPlayer->isVisible());
    Settings::getInstance()->setValue("showPlayerSize", m_showPlayer->size());
    Settings::getInstance()->setValue("showPlayerGeometry", m_showPlayer->saveGeometry());
    Settings::getInstance()->endGroup();
    delete m_showPlayer;
}

void libreMediaServerAudioUi::olasetup()
{
    QWebView *view = new QWebView();
    view->load(QUrl("http://localhost:9090/ola.html"));
    view->show();
}

void libreMediaServerAudioUi::launchShowPlayerWindow()
{
    m_showPlayer->show();
}
